@@ -9,6 +9,7 @@ from typing import (
     Iterable,
     MutableMapping,
     Optional,
+    Container,
 )
 from functools import partial
 
@@ -171,28 +172,79 @@ Identifier = NewType('Identifier', str)
 Identifiers = Union[Iterable[Identifier], str]
 StateFactory = Callable[[Identifiers], StateType]
 
+DFLT_BOUND_VAL_FACTORY = BoundVal
+
+
+class _Binder:
+    _reserved_vars = {'_state', '_factory'}
+
+    def __init__(self, state: StateType, factory=DFLT_BOUND_VAL_FACTORY):
+        self._state = state
+        self._factory = factory
+
+    def __iter__(self):
+        yield from self._state
+
+    __dir__ = __iter__
+
+
+class _DynamicBindsMixin:
+    def __getattr__(self, k):
+        if k not in self.__dict__:
+            setattr(type(self), k, self._factory(k))
+        return getattr(self, k)
+
+    def __setattr__(self, k, v):
+        # Need this "not in _reserved_vars", otherwise the __init__ won't be able to set
+        # _state and _factory
+        if k not in self._reserved_vars:
+            self.__dict__['_state'][k] = v
+        self.__dict__[k] = v  # put it in the __dict__ (so it becomes an attribute)
+
+
+class _ExclusiveBindsMixin(_DynamicBindsMixin):
+    _allowed_ids: Container = ()
+
+    def __getattr__(self, k):
+        if k not in self.__dict__ and k not in self._allowed_ids:
+            raise AttributeError(
+                f'That attribute is not in the self._allowed_ids collection: {k}'
+            )
+        else:
+            return super().__getattr__(k)
+
+    def __setattr__(self, k, v):
+        # Need this "not in _reserved_vars", otherwise the __init__ won't be able to set
+        # _state and _factory
+        if k not in self._reserved_vars and k not in self._allowed_ids:
+            raise ForbiddenWrite(
+                "Can't write there. The id is not in the self._allowed_ids collection"
+                f': {k}'
+            )
+        else:
+            return super().__setattr__(k, v)
+
 
 def mk_binder(
-    *identifiers: Identifiers,
     state: Optional[StateType] = None,
-    bound_val_factory=BoundVal,
+    allowed_ids: Optional[Identifiers] = None,
+    bound_val_factory: Callable = DFLT_BOUND_VAL_FACTORY,
 ):
     """
-
-    :param identifiers:
-    :param bound_val_factory:
-    :return:
-
-    >>> Binder = mk_binder('foo bar')
+    >>> Binder = mk_binder()
     >>> d = dict()
     >>> b = Binder(d)
 
-    We ``b.foo`` exists, but is not set.
+    If I ask for ``b.foo`` (or any valid python identifier I want) it'll be inserted
+    as an "descriptor" attribute of ``Binder``, but it's  value will be special value
+    ``ValueNotSet``.
 
     >>> b.foo
     ValueNotSet
+    >>> 'foo' in dir(Binder)
+    True
 
-    So let's set it:
+    Let's set the value of ``foo```:
 
     >>> b.foo = 42
     >>> b.foo
@@ -225,7 +277,7 @@ def mk_binder(
     A ``Binder`` will also have some useful mapping methods that are linked to the
     underlying ``state``.
 
-    >>> Binder = mk_binder('the', 'variables', 'I', 'want')
+    >>> Binder = mk_binder(allowed_ids=['the', 'variables', 'I', 'want'])
     >>> state = dict()
     >>> b = Binder(state)
     >>> list(b)
@@ -244,41 +296,34 @@ def mk_binder(
     >>> 'variables' in b  # 'variables' not "there" because not set
     False
 
-
     """
-    identifiers = ensure_identifiers(*identifiers)
 
     # TODO: Make it pickalble! (add reduce? Make base outside function?)
-    @dataclass
-    class Binder:
-        _reserved_vars = {'_state', '_factory'}
 
-        def __init__(self, state: StateType, factory=bound_val_factory):
-            self._state = state
-            self._factory = factory
+    if allowed_ids is None:
 
-        def __getattr__(self, k):
-            if k not in self.__dict__:
-                setattr(type(self), k, self._factory(k))
-            return getattr(self, k)
+        class DynamicBinder(_Binder, _DynamicBindsMixin):
+            """Specific Binder with dynamic on-the-fly identifiers"""
 
-        def __setattr__(self, k, v):
-            # Need this "not _state or _factory", or the __init__ won't be able to set
-            # _state and _factory
-            if k not in self._reserved_vars:
-                self.__dict__['_state'][k] = v
-            self.__dict__[k] = v  # put it in the __dict__ (so it becomes an attribute)
+        Binder = DynamicBinder
 
-        def __iter__(self):
-            yield from self._state
+    else:
+        if isinstance(allowed_ids, str):
+            identifiers = ensure_identifiers(allowed_ids)
+        else:
+            identifiers = ensure_identifiers(*allowed_ids)
 
-        # # TODO: Not doing this because:
-        # #  https://docs.python.org/3/library/functions.html#locals
-        # for id_ in identifiers:
-        #     locals()[id_] = bound_val_factory(id_)
+        assert not isinstance(identifiers, str)
 
-    for id_ in identifiers:
-        setattr(Binder, id_, bound_val_factory(id_))
+        class ExclusiveBinder(_Binder, _ExclusiveBindsMixin):
+            """Specific Binder with exclusive identifiers"""
+
+            _allowed_ids = set(identifiers)
+
+        for id_ in identifiers:
+            setattr(ExclusiveBinder, id_, bound_val_factory(id_))
+
+        Binder = ExclusiveBinder
 
     # TODO: Should we really be having the function return type or instance thereof
     #  according to whether state is given?
@@ -286,20 +331,3 @@ def mk_binder(
         return Binder(state=state)
     else:
         return Binder
-
-
-def binder_test():
-    """This work is to try to add 'auto registering' of bounded variables"""
-    Binder = mk_binder()
-    d = dict()
-    s = Binder(d)
-    assert d == {}
-    assert s.foo is ValueNotSet
-    s.foo = 42
-    assert s.foo == 42
-    assert d == {'foo': 42}
-    s.foo = 496
-    assert s.foo == 496
-    assert d == {'foo': 496}
-    s.bar = 8128
-    assert d == {'foo': 496, 'bar': 8128}
